function G = gmatas(theta0,n,d)

%Function to compute local identification condition for the AS(2007) model at
%a point theta0. 
%Inputs: theta0 - vector of  parameters in the order:
%[tau beta kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz,m_eta_r,m_eta_g,m_eta_a,sig_eta]';
%
% n - number of points for evaluating the integral
%Observables order: 1) inflation, 2) output, 3) interest rate

%uses auxiliary files createcov_as.m, dsgesolv.m, gensys_mod.m, svdrr.m

%modification for computing the curve to keep step size constant:
if d==1
    load theta0asi
elseif d==0
    load theta0asd
end

%% Model solution

[TT,TC,TEPS,TETA,RC] = dsgesolv(theta0,1); 
nstate = size(TT,2); %number of state variables


if RC==[1;1] %if determinacy
    RR = [TEPS,zeros(nstate,1)];
    
 
    QQ=createcov_as(theta0); %covariance matrix
    
elseif RC==[1;0] %if indeterminacy
    TETA=rref(TETA').'; %reduced column echelon form for TETA
    RR = [TEPS,TETA]; 
    QQ=createcov_as(theta0); %augmented covariance matrix
else
    G=[];
    disp('Nonexistence or numerical problems')
    return
end
%% Spectrum computation
%set up selection matrix
A=[1	0	0	0	0	0	0;
   0	1	0	0	0	0	0;
   0	0	1	0	0	0	0]; 



%the integral [-pi,pi] is approximated using n points
%n=10000;
omi=2*pi*(-(n/2-1):1:(n/2))'/n; %form vector of Fourier frequencies for approximation of the integral in G
dimy=size(A,1); %number of observables
sp_mat=zeros(dimy*n,dimy); %blank for spectra at all frequencies in omi
for i=1:n
exe=exp(-1i*omi(i));
%A(2,2)=(1-exe);
mat1=(eye(nstate)-TT*exe)\eye(nstate); %inv(1-T1L)
mat2=mat1';  %note that ' gives the conjugate transpose

sp_mat(((i-1)*dimy+1):(i*dimy),:)=A*mat1*RR*QQ*RR'*mat2*A'/(2*pi);

end
%% Numerical derivative
nsp_mat=zeros(dimy*n,dimy*length(theta0)); %blank for derivatives
dif_para=theta*1e-7; %differentiation step size

for j=1:length(theta0);

%perturb one parameter at a time
npara=theta0;
npara(j)=theta0(j)-dif_para(j);

%recompute the spectral density matrix)
[nT1,nTC,nTEPS,nTETA,nRC] = dsgesolv(npara,1);

if nRC==[1;1]; % determinacy 

   nRR = [nTEPS,zeros(nstate,1)];
   nQQ=createcov_as(npara); %covariance matrix
   
elseif nRC==[1;0]; % indeterminacy 
   nTETA=rref(nTETA').'; %reduced column echelon form for nTETA
   nRR = [nTEPS,nTETA]; 
   nQQ=createcov_as(npara); %augmented covariance matrix
else
   G=[];
   disp('Nonexistence or numerical problems - cannot compute derivative')
   return
   
end



for i=1:n;
exe=exp(-1i*omi(i));
%A(2,2)=(1-exe);
nmat1=(eye(nstate)-nT1*exe)\eye(nstate);
nmat2=nmat1';

nsp_mat(((i-1)*dimy+1):(i*dimy),((j-1)*dimy+1):(j*dimy))=A*nmat1*nRR*nQQ*nRR'*nmat2*A'/(2*pi); %spectra at perturbed theta0

end
end

%now construct the G matrix, note that the jkth component is simply the derivative with respect to j and k
%a simple two-point method is used

Gmat=zeros(length(theta0),length(theta0)); %blank

for j=1:length(theta0)

for k=1:length(theta0)
aj=(sp_mat-nsp_mat(:,((j-1)*dimy+1):(j*dimy)))/dif_para(j);
ak=(sp_mat-nsp_mat(:,((k-1)*dimy+1):(k*dimy)))/dif_para(k);

for freq=1:n;
Gmat(j,k)=Gmat(j,k)+sum(diag(aj(((freq-1)*dimy+1):(freq*dimy),:)*ak(((freq-1)*dimy+1):(freq*dimy),:)));

end

end

end
Gmat=2*pi*Gmat/n;
G=real(Gmat); 


end