%******************************************************************************
%*      Solve the AS(2007) DSGE  model using GENSYS
%
%       msel=1: current inflation, output gap rule
%       msel=2: expected inflation, output gap rule
%       msel=3: current inflation, output growth rule 
%**************************************************************************/
function [T1,TC,T0,TETA,RC] = dsgesolv(para,msel)

%theta=[tau beta kappa psi1 psi2 rhoR rhog rhoz sigR  sigg sigz m_eta_r m_eta_g m_eta_z sig_eta];

%********************************************************************
%**      assign names to parameters
%********************************************************************
tau        = para(1);

kap        = para(3);
psi1       = para(4);
psi2       = para(5);
rhoR       = para(6);
rhog       = para(7);
rhoz       = para(8);

%** additional parameters **
bet =para(2);

%********************************************************************
%**      matrices of canonical system
%********************************************************************

%** equation indices **
eq_euler = 1;  % Euler equation **
eq_p     = 2;  % Phillip's Curve **
eq_mp    = 3;  % Monetary policy rule **
eq_g     = 4;  % g process **
eq_z     = 5;  % z process **
eq_Epi   = 6;  % E(pi) **
eq_Ey    = 7;  % E(y) **


%** variable indices **
v_pi  = 1;
v_y   = 2;
v_R   = 3;
v_g   = 4;
v_z   = 5;
v_Epi = 6;
v_Ey  = 7;

%** shock indices (neps) **
e_R = 1;
e_g = 2;
e_z = 3;

%** expectation error indices (neta) **
n_pi = 1;
n_y  = 2;

%** summary **
neq  = 7;
neps = 3;
neta = 2;

%** initialize matrices **
GAM0 = zeros(neq,neq);
GAM1 = zeros(neq,neq);
   C = zeros(neq,1);
 PSI = zeros(neq,neps);
 PPI = zeros(neq,neta);

%**********************************************************
%**      1. Euler Equation
%**********************************************************
GAM0(eq_euler,v_y)   = -1;
GAM0(eq_euler,v_Ey)  =  1;
GAM0(eq_euler,v_R)   = -1/tau;
GAM0(eq_euler,v_Epi) =  1/tau;
GAM0(eq_euler,v_g)   =  1-rhog;
GAM0(eq_euler,v_z)   =  rhoz/tau;

%**********************************************************
%**      2. Phillip's Curve
%**********************************************************
GAM0(eq_p,v_pi)  = -1;
GAM0(eq_p,v_Epi) =  bet;
GAM0(eq_p,v_y)   =  kap;
GAM0(eq_p,v_g)   = -kap;

%**********************************************************
%**      3. Monetary policy rule
%**********************************************************

if msel == 1; %* current inflation, output gap rule *

  GAM0(eq_mp,v_R)  =   1;
  GAM0(eq_mp,v_pi) = -(1-rhoR)*psi1;
  GAM0(eq_mp,v_y)  = -(1-rhoR)*psi2;
  GAM0(eq_mp,v_g)  = (1-rhoR)*psi2;    

  GAM1(eq_mp,v_R)  =   rhoR;

   PSI(eq_mp,e_R)  =   1;
   
elseif msel==2 %expected inflation, output gap rule
  GAM0(eq_mp,v_R)  =   1;
  GAM0(eq_mp,v_Epi) = -(1-rhoR)*psi1; %expected inflation
  GAM0(eq_mp,v_y)  = -(1-rhoR)*psi2;
  GAM0(eq_mp,v_g)  = (1-rhoR)*psi2;    

  GAM1(eq_mp,v_R)  =   rhoR;

   PSI(eq_mp,e_R)  =   1;
   
elseif msel==3 %* current inflation, output growth rule *

  GAM0(eq_mp,v_R)  =   1;
  GAM0(eq_mp,v_pi) = -(1-rhoR)*psi1;
  GAM0(eq_mp,v_y)  = -(1-rhoR)*psi2;
  GAM0(eq_mp,v_z)  = -(1-rhoR)*psi2;    

  GAM1(eq_mp,v_R)  =   rhoR;
  GAM1(eq_mp,v_y)  = -(1-rhoR)*psi2;   

   PSI(eq_mp,e_R)  =   1;
   

end

%**********************************************************
%**      Shock process
%**********************************************************
%** g **
GAM0(eq_g,v_g) = 1;
GAM1(eq_g,v_g) = rhog;
 PSI(eq_g,e_g) = 1;

%** z **
GAM0(eq_z,v_z) = 1;
GAM1(eq_z,v_z) = rhoz;
 PSI(eq_z,e_z) = 1;

%**********************************************************
%**      Expectation error
%**********************************************************
%** E[pi] **
GAM0(eq_Epi,v_pi)  = 1;
GAM1(eq_Epi,v_Epi) = 1;
 PPI(eq_Epi,n_pi)  = 1;
%** E[y] **
GAM0(eq_Ey,v_y)  = 1;
GAM1(eq_Ey,v_Ey) = 1;
 PPI(eq_Ey,n_y)  = 1;


%********************************************************************
%**      QZ(generalized Schur) decomposition by GENSYS
%********************************************************************
  [T1,TC,T0,TY,M,TZ,TETA,GEV,RC]=gensys_mod(GAM0, GAM1, C, PSI, PPI, 1); %use Sims-based code, also uses ordqz
    


end
