function lp = logprasn(theta)
%Computes the log prior for AS07 model. 
%Dimension of theta is 15x1 (last 4 parameters are sunspot parameters)
%Some code here is borrowed from An and Schorfheide's (2007)
%logprior.m file

%parameterization should be: tau; r-star instead of
%beta, and standard deviations instead of variances.

%parameter order:
%[tau rstar kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz, m_eta_r, m_eta_g, m_eta_z, sig_eta]';

n  = length(theta);
 
theta0=partrans(theta,2);

[TT,TC,TEPS,TETA,RC] = dsgesolv(theta0,1);

%index priors by 1 - Beta, 2 - Gamma, 3 - Normal, 4 - Invgamma
priorind=[2,2,2,2,2,1,1,1,4,4,4,3,3,3,4]'; %LS(2004) prior densities

%Use Lubik-Schorfheide (2004) prior 1 specification. Note the first value equals the mean, the second the std. dev.
%except for inverse gamma, for which the two values are parameters related to mean
%and std. dev. - see footnote in LS(2004) Table 1
%from LS table 1
asparm=[2,0.5;2,1;0.5,0.2;1.1,0.5;0.25,0.15;0.5,0.2;0.7,0.1;0.7,0.1;0.25,4;0.30,4;0.6,4;0,1;0,1;0,1;0.2,4]; 

if RC==[1;0] %indeterminacy
y = 0;
a = 0;
b = 0;
for i=1:n

	shape = priorind(i); 
	para1 = asparm(i,1);
	para2 = asparm(i,2);

	% BETA prior
	if shape==1
		a = (1-para1)*para1^2/para2^2 - para1;
		b = a*(1/para1 - 1);
		y = y + lnpdfbeta(theta(i),a,b);

	% GAMMA prior
	elseif shape==2
		b = para2^2/para1;
		a = para1/b;
		y = y + lnpdfgam(theta(i),a,b);

	% NORMAL prior
	elseif shape==3
		a = para1;
		b = para2;
		y = y + lnpdfnorm(theta(i),a,b);

	% INVGAMMA prior
	elseif shape==4
		a = para1;
		b = para2;
		y = y + lnpdfig(theta(i),a,b);

    end
    lp=-y; %minus the log prior
end

elseif RC==[1;1] %determinacy
y = 0;
a = 0;
b = 0;
for i=1:n

	shape = priorind(i); 
	para1 = asparm(i,1);
	para2 = asparm(i,2);

	% BETA prior
	if shape==1
		a = (1-para1)*para1^2/para2^2 - para1;
		b = a*(1/para1 - 1);
		y = y + lnpdfbeta(theta(i),a,b);

	% GAMMA prior
	elseif shape==2
		b = para2^2/para1;
		a = para1/b;
		y = y + lnpdfgam(theta(i),a,b);

	% NORMAL prior
	elseif shape==3
		a = para1;
		b = para2;
		y = y + lnpdfnorm(theta(i),a,b);

	% INVGAMMA prior
	elseif shape==4
		a = para1;
		b = para2;
		y = y + lnpdfig(theta(i),a,b);

    end
    lp=-y; %minus the log prior density
end

else %no equilibrium/problems
    lp=[]; %assign empty value

end

end



%
% logs of density functions
%
%These functions are borrowed from AS(2007)

% beta
function y=lnpdfbeta(x,a,b)
% LNPDFBETA(X,A,B)
%	calculates log BETA(A,B) at X
% Sungbae An
% 03/03/2002
y = gammaln(a+b) - gammaln(a) - gammaln(b) + (a-1)*log(x) + (b-1)*log(1-x);
end

% gamma
function y=lnpdfgam(x,a,b)
% LNPDFGAM(X,A,B)
%	calculates log GAMMA(A,B) at X
% 03/03/2002
% Sungbae An
y = -gammaln(a) -a*log(b)+ (a-1)*log(x) -x/b ;
end

% inverse gamma
function y = lnpdfig(x,a,b)
% LNPDFIG(X,A,B)
%	calculates log INVGAMMA(A,B) at X
% 03/03/2002
% Sungbae An
y = log(2) - gammaln(b/2) + (b/2)*log(b*a^2/2) - ( (b+1)/2 )*log(x^2) - b*a^2/(2*x^2);
end

% normal
function y = lnpdfnorm(x,a,b)
% LNPDFNORM(X,A,B)
%	calculates log NORMAL(A,B) at X
% 03/03/2002
% Sungbae An
y = -0.5*log(2*pi) - log(b) - 0.5*(x-a)^2/b^2;
end