%%% This file reproduces the results in the eighth column of Tables 2 and 3:
%%% i) the parameter vector minimizing the KL criterion between the
%%% default specification of the An and Schorfheide (2007) model under
%%% indeterminacy and the parameters in the space outside the excluded 
%%% neighborhood with c=0.5. Parameters tau and psi2 are held fixed at the 
%%% default values of 2.51 and 0.23 respectively.
%%% 
%%% ii) the value of the minimized KL criterion and the empirical distances 
%%% for samples of size 80, 150, 200 and 1000.

% The required data files and subroutines are located in: 
%\AS_Identification\
%(dsgesolv.m, klas.m,pfhas.m,qas.m,vfhas.m, theta0asi.mat,true_spectrum0asi.mat)
%Additional files for  business cycle frequencies only:
%(klasbc.m,pfhasbc.m,qasbc.m,vfhasbc.m,true_spectrum0asibc.mat)
%\AS_Identification\Constraints\
%(constraintastp.m)
%\AS_Identification\Objectives\
%(kloptas3.m)
%\General\gensys_mod
%(gensys_mod.m, svdrr.m)
%\General\GA_optim.m, PSO_optim.m, psout.m

%The total runtime for GA+Multistart algorithm is about 6.5 minutes, and about 8 minutes for PSO+Multistart  
%algorithm on a Xeon E5-2665 8-core 2.4Ghz processor, using Matlab R2015a.
%%
clear
%% Algorithm selection and Matlab versions
%Select one procedure:
runga=0; %1 if using Genetic algorithm + Multistart combination, 0 otherwise
runpso=1; %1 if using Particle Swarm + Multistart combination, 0 otherwise
          %-----Note PSO is available only in R2014b and later----
 
%Specify the constraint handling for the Genetic algorithm.
NonlinCon='penalty'; %this is the preferred option, available in the matlab version 2014b and later; for earlier versions, set to 'auglag'.
%% Set up parallel computation

 numcore=8; %specify the number of cores
% 
% %for versions before R2014a
% matlabpool open local numcore
% 
% %for versions R2014a and above
ppool=parpool('local',numcore); %create parallel pool object
%% Load parameter values and bounds

load theta0asi %load default parameter vector (theta0)

% Set lower and upper bounds
%Parameter order: 
%[tau beta kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz, m_eta_r, m_eta_g, m_eta_z, sig_eta]

lb=[0.01 0.9 0.01 0.01  0.01  0.1  0.1  0.1  0.01  0.01 0.01  -3 -3   -3    0.001];

ub=[10   0.999   5  0.9 5 0.99 0.99 0.99   3   3    3      3   3  3 3];

%exclude parameters tau and psi2
lb([1,5])=[];
ub([1,5])=[];

%% Select frequencies 
bc=0; %0 for full spectrum; 1 for business cycle frequencies only

%% Specify constraints 
ns=0.5; %size of excluded neighborhood; only needed for constrained case.

nrm=Inf; %set norm for the constraint function (1,2, or Inf)

numpar=length(lb);%number of parameters in the objective function

indp=[1:numpar]; %vector of parameter indices to be constrained
wgt=ones(1,length(indp)); %vector of weights for the constraint function

%% Detailed specifications; you should not have to modify them

bc=0; %0 for full spectrum; 1 for business cycle frequencies only
if bc==0
    n=100; %number of points to evaluate the integral
    w=2*pi*(-(n/2-1):1:n/2)'/n; %form vector of Fourier frequencies
    resfilename=['tables_23_p8'];
elseif bc==1
    n=500; %number of points to evaluate the integral
    w=2*pi*(-(n/2-1):1:n/2)'/n; %form vector of Fourier frequencies
    resfilename=['tables_23_p8_bc'];
end

con1=[0,ns]; %constraint handling: con(1)=0 if algorithm handles constraint,
%con(1)=1 if penalty to be added to the objective function for infeasible
%pioints. con(2) passes the excluded neighborhood size to the constraint
%function.

con2=[1,ns]; %set constraint handling for Particle Swarm optimization.


ObjectiveFunction = @(theta0)kloptas3(theta0,con1,w,wgt,nrm,indp,bc); %set objective function for GA/multistart
ObjectiveFunctionP = @(theta0)kloptas3(theta0,con2,w,wgt,nrm,indp,bc); %set penalized objective function for PSO
ConstraintFunction=@(xest)constraintastp(xest,ns,wgt,nrm,indp); %set constraint for the problem

dispalg='iter'; %set whether algorithm iterations are displayed.
dispint=20; %interval between displayed iterations (for PSO)

%% GA algorithm settings
if runga==1
gen=1000; %max number of generation for GA
stgenlim=50; %max number of stall generations (i.e., no improvement found)

initpop=[]; %set initial population (if smaller than popsize, MATLAB will
%randomly draw the rest. If [], the whole population is randomly drawn.
%Can be a row vector of dimension numpar or a matrix. Each row of the
%matrix is then a candidate initial value.

popsize=100; %population size
elcnt=3; %elite count - number of elite individuals retained in population

tolfunga=1e-10; %tolerance level for improvement in the objective for GA
tolconga=1e-10; %tolerance level for constraint for GA

usepga=['Always']; %Set to 'Always' to use parallel computation, otherwise to 'Never' or []
%In later versions of Matlab, 1 and 0 can also be used respectively. 
end

%% PSO algorithm settings
if runpso==1
    
swarmsize=300; %swarm size (similar concept to population size for GA)
maxitpso=1000; %maximum number of iterations (similar concept to generations for GA)
stiterlim=100; %max number of stall PSO iterations (i.e., no improvement found)
initswarm=[]; %set initial population (if smaller than swarmsize, MATLAB will
%randomly draw the rest. If [], the whole population is randomly drawn.
%Can be a row vector of dimension numpar or a matrix. Each row of the
%matrix is then a candidate initial value.

minfn=0.1; %smallest fraction of neighbors for PSO (smallest size of the 
%adaptive neighborhood)

tolfunpso=1e-06; %tolerance level for improvement in the objective for PSO

psoname=['psoastp_c',num2str(ns*100)]; %set name for a temp output file that stores the swarms (problem-based)
OutFun=@(optimValues,state)psout(optimValues,state,psoname); %output 
%function for extracting swarms from PSO runs for further local optimization.

useppso=['Always']; %Set to 'Always' to use parallel computation, otherwise to 'Never' or []


end
%% Multistart algorithm settings

numrpoints=50; %number of random starting points for Multistart

usepms=['Always']; %Set to 'Always' to use parallel computation, otherwise to 'Never' or []

% settings for fmincon
maxit=1000; % set max number of iterations
maxfev=10000; % set max number of function evaluations
tolfunfmc=1e-10; %tolerance level for improvement in the objective for fmincon
tolconfmc=1e-10; %tolerance level for constraint for fmincon
tolx=1e-10; %tolerance on solution value

localg='active-set'; %set main local algorithm to be used for multistart

%% Run optimization

if runga==1
    timega=tic;
    GA_optim %run GA+Multistart
    timelga=toc(timega); %time taken by GA/Multistart
    save(resfilename) %save intermediate results
end
save(resfilename)
if runpso==1
    timepso=tic;
    PSO_optim %run PSO+Multistart
    timelpso=toc(timepso); %time taken by PSO/Multistart
    save(resfilename) %save intermediate results
end



%% Arrange results
values=[]; %blank for storing best function values
solvecs=[]; %blank for stroing solution vectors
if runga==1
    values=[values;fvalga;fvalga2];
    solvecs=[solvecs;xestga;xestga2];
end
if runpso==1
    values=[values;fvalpso;fvalpso2];
    solvecs=[solvecs;xestpso;xestpso2];
end

err=find(values<0);
values(err)=1e07; %penalize negative values that may occur due to algorithm error
indm=find(values==min(values)); %minimum value(s)
indm=indm(1); 
temp1=['thetaind=[theta(1),solvecs(',num2str(indm),',(1:3)),theta(5),solvecs(',num2str(indm),',(4:end))]'';'];
temp2=['kl=values(',num2str(indm),')/10000;']; 
eval(temp1); %save final parameter result
eval(temp2); %save minimized KL distance
%%
%% Empirical distance computation
if bc==0
    ed=[pfhas(theta,thetaind,1,1,0.05,80);pfhas(theta,thetaind,1,1,0.05,150);pfhas(theta,thetaind,1,1,0.05,200);pfhas(theta,thetaind,1,1,0.05,1000)];
elseif bc==1
    ed=[pfhasbc(theta,thetaind,1,1,0.05,80);pfhasbc(theta,thetaind,1,1,0.05,150);pfhasbc(theta,thetaind,1,1,0.05,200);pfhasbc(theta,thetaind,1,1,0.05,1000)];
end
%% Print and save results
par=['tau     '; 'beta    '; 'kappa   '; 'psi1    '; 'psi2    '; 'rhoR    '; 'rhog    '; 'rhoz    '; 'sigR    '; 'sigg    '; 'sigz    ';'m_eta_r '; 'm_eta_g '; 'm_eta_z '; 'sig_eta '];
t0=num2str(theta,3);
ti=num2str(thetaind,3);
t3=['KL    ';'T=80  ';'T=150 ';'T=200 ';'T=1000'];

for i=1:15
    sp(i,:)='  ';
end
disp 'Table 2. Parameter values minimizing the KL criterion, AS (2007) model'
disp '        (c) tau and psi2 fixed'
disp '           theta0     c=0.5'
disp([par,sp,t0,sp,ti])

disp 'Table 3. KL and empirical distances between theta_c and theta_0, AS (2007) model'
disp '        (c) tau and psi2 fixed'
disp '           c=0.5'
disp([t3,sp(1:5,:),num2str([kl;ed],3)])
save(resfilename)
%% Close parallel pool

% %for versions before R2014a
% matlabpool close
% 
% %for versions R2014a and above
delete(ppool) %delete the parallel pool object